@@ -1,6 +1,5 @@
 import asyncio
 import functools
-import sys
 import unittest
 from unittest import mock
 
@@ -12,7 +11,6 @@ from socketio import packet
 from .helpers import AsyncMock, _run
 
 
-@unittest.skipIf(sys.version_info < (3, 5), 'only for Python 3.5+')
 class TestAsyncPubSubManager(unittest.TestCase):
     def setUp(self):
         id = 0
@@ -541,7 +539,6 @@ class TestAsyncPubSubManager(unittest.TestCase):
                    'host_id': host_id}
             yield pickle.dumps({'method': 'close_room', 'value': 'baz',
                                 'host_id': host_id})
-            raise asyncio.CancelledError()  # force the thread to exit
 
         self.pm._listen = messages
         _run(self.pm._thread())
